// © 2023, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

#pragma once

#include "src/math/Vec4f.h"

// Forward declaration of the shader:
class Shader;

/**
 * The result of an intersection test (returned by the 'intersect'-function of a surface).
 *
 * If 'hasIntersection' is false, the value of the other parameters are undefined.
 */
struct Intersection
{
    /** True if a surface was hit by the ray, otherwise false */
    bool hasIntersection = false;

    /** Parameter of the intersection point (hitPoint = origin + direction * t) */
    float t = FLT_MAX;

    /** Intersection point in world space */
    Vec4f hitPoint;

    /** Normal of the surface at the intersection point */
    Vec4f normal;

    /** The shader that should be used at that intersection point */
    Shader* shader = nullptr;

    /** Overwrite this intersection with other intersection, in case it's closer */
    void combineClosest( Intersection other ) {
        if (other.hasIntersection && other.t < t)
            *this = other;
    }
};
