#include "Ray.h"

Ray::Ray(){
    origin = Vec4f();
    origin = Vec4f(0.f, 0.f, 0.f, 0.f);
}

Ray::Ray(Vec4f pOrigin, Vec4f pDirection)
{
    origin = pOrigin;
    direction = pDirection;
}

Ray Ray::reflected(Vec4f hitPoint, Vec4f surfaceNormal){
    Vec4f reflectedDir = direction - surfaceNormal * surfaceNormal.dot(direction) * 2.f;
    return Ray(hitPoint, reflectedDir);
}

Ray Ray::refracted(Vec4f hitPoint, Vec4f surfaceNormal, float refractiveIndex){
    Vec4f dir = direction.normalized();
    Vec4f n = surfaceNormal;

    // Calculate the angle:
    float cos1 = surfaceNormal.dot(-dir);

    float r = refractiveIndex;

    // If the angle is negative, the normal is not directed the air side.
    if (cos1 > 0)
        r = 1 / r;
    else {
        n = -n;
        cos1 = -cos1;
    }

    float cos2 = 1.f - (1.f - cos1 * cos1) * (r * r);

    if (cos2 < 0)
         return Ray();

    return Ray(hitPoint, (dir + n * cos1)*r-n*sqrt(cos2));
}

Vec4f Ray::evaluate(float t) const {
    return origin + direction * t;
}

bool Ray::isValid(){
    return !direction.isNull() && direction.valid();
}
