// © 2023, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

#pragma once

#include "src/math/Mat4f.h"

#include "src/raytracer/Ray.h"

/**
 * Represents a simple pinhole camera.
 */

class Camera {
    /** The transformation matrix of the camera, which positions the camera into the world */
    Mat4f model;

    float mouseX = 0.0f;
    float mouseY = 0.0f;

    float distance = 3.f;

    Vec4f target;
    float focalLength;

    void updateModelMatrix();

public:
    /**
     *
     */
    Camera();

    /**
     * Initializes the camera.
     */
    Camera(Vec4f target, float distance, float focalLength, float xRotation = 0, float yRotation = 0);

    /**
     * Generates a ray that goes through the given pixel (x,y) for an image with the given size
     * using the camera settings of this object.
     *
     * @param x             x position of pixel.
     * @param y             y position of pixel.
     * @param imageWidth    width of the image to be rendered on.
     * @param imageHeight   height of the image to be rendered on.
     *
     * @return A ray that goes through the given pixel (x,y).
     */
    Ray generateRay(unsigned int x, unsigned int y, unsigned int imageWidth, unsigned int imageHeight);

    /**
     *
     * @param mouseDeltaX
     * @param mouseDeltaY
     */
    void rotateCamera(float mouseDeltaX, float mouseDeltaY);

    void multiplyDistance(float multiplier);
};
