// © 2023, CGVR (https://cgvr.informatik.uni-bremen.de/),
// Author: Andre Mühlenbrock (muehlenb@uni-bremen.de)

#pragma once

#include <vector>
#include <memory>

#include "src/scene/Surface.h"
#include "Accelerator.h"

enum AcceleratorType
{
    NoneType,
    UniformGridType,
    AabbTreeType,
    SphereTreeType,
    KdTreeType,
    OctreeType
};

/**
 * A surface that consists of multiple surfaces (inherits from Surface).
 */
class SurfaceList : public Surface {
    std::vector<std::shared_ptr<Surface>> surfaces;
    AcceleratorType acceleratorType = AcceleratorType::NoneType;
    Accelerator *accelerator = nullptr;
    Aabb aabb;

public:
    /**
     * Adds a surface to this surface list.
     */
    void addSurface(std::shared_ptr<Surface> surface);

    /**
     * Adds a surface to this surface list.
     */
    void addSurfaces(std::vector<std::shared_ptr<Surface>> pSurfaces);

    /**
     * See documentation in the parent Surface class.
     */
    virtual Intersection intersect(Ray ray, float t0, float t1) override;

    void setAccelerator( AcceleratorType acc );
    void initializeAccelerator( AcceleratorType acc );
    void destroyAccelerator();

    virtual Aabb getAabb() const override;

    std::vector<std::shared_ptr<Surface>> getSurfaces() const;
};
