#include "Shader.h"

Vec4f Shader::shade(PointLight& light, Intersection& intersection, Ray& ray){
    Vec4f normal = intersection.normal;
    Vec4f ingoingDir = light.getPosition() - intersection.hitPoint;
    Vec4f outgoingDir = -ray.direction;

    float cosTheta = normal.dot(ingoingDir);

    if(cosTheta < 0)
        return Vec4f(0.f, 0.f, 0.f);

    Vec4f halfway = (ingoingDir + outgoingDir).normalized();

    float highlight = pow(normal.dot(halfway), shininess);

    Vec4f attenuatedLightColor = light.getColor() * (1 / ingoingDir.length());

    return attenuatedLightColor * (diffuseColor * cosTheta + reflectiveColor * highlight);
}

Vec4f Shader::shadeAmbient(Vec4f color){
    return color * ambientColor;
}

Vec4f Shader::getAmbientColor(){
    return ambientColor;
}

Vec4f Shader::getReflectiveColor(){
    return reflectiveColor;
}

Vec4f Shader::getRefractiveColor(){
    return refractiveColor;
}

float Shader::getRefractiveIndex(){
    return refractiveIndex;
}

bool Shader::hasReflective(){
    return reflectiveColor.dot(reflectiveColor) > 0.f;
}

bool Shader::hasRefractive(){
    return refractiveColor.dot(refractiveColor) > 0.f;
}
